"""Diagnostic pages for genetics-viz."""

from genetics_viz.pages.diagnostic.all import diagnostic_all_page
from genetics_viz.pages.diagnostic.statistics import diagnostic_statistics_page

__all__ = [
    "diagnostic_all_page",
    "diagnostic_statistics_page",
]
