"""Components for cohort pages."""

from genetics_viz.pages.cohort.components.dnm_tab import render_dnm_tab
from genetics_viz.pages.cohort.components.svs_tab import render_svs_tab
from genetics_viz.pages.cohort.components.wombat_tab import render_wombat_tab

__all__ = ["render_wombat_tab", "render_dnm_tab", "render_svs_tab"]
